// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { Identifier } from "./Identifier";

/**
 * PRAGMA statement (SQLite)
 */
export type Pragma = { 
/**
 * Optional schema prefix (e.g., "schema" in "schema.pragma_name")
 */
schema: Identifier | null, 
/**
 * The pragma name
 */
name: Identifier, 
/**
 * Optional value for assignment (PRAGMA name = value)
 */
value: Expression | null, 
/**
 * Optional arguments for function-style pragmas (PRAGMA name(arg))
 */
args: Array<Expression>, };
