// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Identifier } from "./Identifier";

/**
 * Column position for ADD COLUMN (MySQL/MariaDB)
 */
export type ColumnPosition = "First" | { "After": Identifier };
