// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { Identifier } from "./Identifier";

/**
 * CACHE TABLE statement (Spark)
 */
export type Cache = { 
/**
 * The table to cache
 */
table: Identifier, 
/**
 * LAZY keyword - defer caching until first use
 */
lazy: boolean, 
/**
 * Optional OPTIONS clause (key-value pairs)
 */
options: Array<[Expression, Expression]>, 
/**
 * Optional AS clause with query
 */
query: Expression | null, };
