// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Identifier } from "./Identifier";
import type { Over } from "./Over";

/**
 * Named window definition (WINDOW w AS (...))
 */
export type NamedWindow = { name: Identifier, spec: Over, };
