// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * Oracle CONNECT_BY_ROOT function - returns root row's column value
 */
export type ConnectByRoot = { this: Expression, };
