// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * RowFormatDelimitedProperty
 */
export type RowFormatDelimitedProperty = { fields: Expression | null, escaped: Expression | null, collection_items: Expression | null, map_keys: Expression | null, lines: Expression | null, null: Expression | null, serde: Expression | null, };
