// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * Represent a QUALIFY clause for filtering on window function results.
 *
 * Supported by Snowflake, BigQuery, DuckDB, and Databricks. The predicate
 * typically references a window function (e.g.
 * `QUALIFY ROW_NUMBER() OVER (PARTITION BY id ORDER BY ts DESC) = 1`).
 */
export type Qualify = { 
/**
 * The filter predicate over window function results.
 */
this: Expression, };
