// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * JSON_SET / JSON_INSERT function
 */
export type JsonModifyFunc = { this: Expression, path_values: Array<[Expression, Expression]>, };
