// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * Represent the FROM clause of a SELECT statement.
 *
 * Contains one or more table sources (tables, subqueries, table-valued
 * functions, etc.). Multiple entries represent comma-separated implicit joins.
 */
export type From = { 
/**
 * The table source expressions.
 */
expressions: Array<Expression>, };
