// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { DataType } from "./DataType";
import type { Expression } from "./Expression";

/**
 * Struct field with optional OPTIONS clause (BigQuery) and COMMENT (Spark/Databricks)
 */
export type StructField = { name: string, data_type: DataType, options?: Array<Expression>, comment?: string | null, };
