// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * MAKE_TIMESTAMP function
 */
export type MakeTimestampFunc = { year: Expression, month: Expression, day: Expression, hour: Expression, minute: Expression, second: Expression, timezone: Expression | null, };
