// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Ordered } from "./Ordered";

/**
 * SORT BY clause (Hive/Spark)
 * Sorts data within each reducer (local sort, not global)
 */
export type SortBy = { expressions: Array<Ordered>, };
