// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * LimitOptions
 */
export type LimitOptions = { percent: Expression | null, rows: Expression | null, with_ties: Expression | null, };
