// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * Index
 */
export type Index = { this: Expression | null, table: Expression | null, unique: boolean, primary: Expression | null, amp: Expression | null, params: Array<Expression>, };
