// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Identifier } from "./Identifier";

/**
 * Trigger event (INSERT, UPDATE, DELETE, TRUNCATE)
 */
export type TriggerEvent = "Insert" | { "Update": Array<Identifier> | null } | "Delete" | "Truncate";
