// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * IF function
 */
export type IfFunc = { condition: Expression, true_value: Expression, false_value: Expression | null, 
/**
 * Original function name (IF, IFF, IIF) for round-trip preservation
 */
original_name?: string | null, };
