// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { Identifier } from "./Identifier";

/**
 * UNNEST function
 */
export type UnnestFunc = { this: Expression, 
/**
 * Additional arguments for multi-argument UNNEST (e.g., UNNEST(arr1, arr2))
 */
expressions?: Array<Expression>, with_ordinality: boolean, alias: Identifier | null, 
/**
 * BigQuery: offset alias for WITH OFFSET AS <name>
 */
offset_alias?: Identifier | null, };
