// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Function security (DEFINER, INVOKER, or NONE)
 */
export type FunctionSecurity = "Definer" | "Invoker" | "None";
