// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * TSQL temporal column constraint: GENERATED ALWAYS AS ROW START|END [HIDDEN]
 */
export type GeneratedAsRow = { 
/**
 * true = ROW START, false = ROW END
 */
start: boolean, 
/**
 * HIDDEN modifier
 */
hidden: boolean, };
