// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * Function body
 */
export type FunctionBody = { "Block": string } | { "StringLiteral": string } | { "Expression": Expression } | { "External": string } | { "Return": Expression } | { "Statements": Array<Expression> } | { "DollarQuoted": { content: string, tag: string | null, } };
