// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * TruncateTable
 */
export type TruncateTable = { expressions: Array<Expression>, is_database: Expression | null, exists: boolean, only: Expression | null, cluster: Expression | null, identity: Expression | null, option: Expression | null, partition: Expression | null, };
