// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * Search
 */
export type Search = { this: Expression, expression: Expression, json_scope: Expression | null, analyzer: Expression | null, analyzer_options: Expression | null, search_mode: Expression | null, };
