// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * MergeTreeTTL
 */
export type MergeTreeTTL = { expressions: Array<Expression>, where_: Expression | null, group: Expression | null, aggregates: Expression | null, };
