// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { DataType } from "./DataType";
import type { Identifier } from "./Identifier";

/**
 * Type attribute for composite types
 */
export type TypeAttribute = { name: Identifier, data_type: DataType, collate: Identifier | null, };
