// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { Ordered } from "./Ordered";

/**
 * GROUP_CONCAT function (MySQL style)
 */
export type GroupConcatFunc = { this: Expression, separator: Expression | null, order_by: Array<Ordered> | null, distinct: boolean, filter: Expression | null, };
