// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Alias } from "./Alias";
import type { Identifier } from "./Identifier";

/**
 * Represent a wildcard star expression (`*`, `table.*`).
 *
 * Supports the EXCEPT/EXCLUDE, REPLACE, and RENAME modifiers found in
 * DuckDB, BigQuery, and Snowflake (e.g. `SELECT * EXCEPT (id) FROM t`).
 */
export type Star = { 
/**
 * Optional table qualifier (e.g. `t` in `t.*`).
 */
table: Identifier | null, 
/**
 * EXCLUDE / EXCEPT columns (DuckDB, BigQuery, Snowflake)
 */
except: Array<Identifier> | null, 
/**
 * REPLACE expressions (BigQuery, Snowflake)
 */
replace: Array<Alias> | null, 
/**
 * RENAME columns (Snowflake)
 */
rename: Array<[Identifier, Identifier]> | null, 
/**
 * Trailing comments that appeared after the star
 */
trailing_comments?: Array<string>, };
