// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * JSONTable
 */
export type JSONTable = { this: Expression, schema: Expression | null, path: Expression | null, error_handling: Expression | null, empty_handling: Expression | null, };
