// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * IS TRUE / IS FALSE predicate
 */
export type IsTrueFalse = { this: Expression, not: boolean, };
