// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { ExecuteParameter } from "./ExecuteParameter";
import type { Expression } from "./Expression";

/**
 * EXEC/EXECUTE statement (TSQL stored procedure call)
 * Syntax: EXEC [schema.]procedure_name [@param=value, ...]
 */
export type ExecuteStatement = { 
/**
 * The procedure name (can be qualified: schema.proc_name)
 */
this: Expression, 
/**
 * Named parameters: @param=value pairs
 */
parameters: Array<ExecuteParameter>, };
