// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Kind of USE statement
 */
export type UseKind = "Database" | "Schema" | "Role" | "Warehouse" | "Catalog" | "SecondaryRoles";
