// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { Ordered } from "./Ordered";
import type { WindowFrame } from "./WindowFrame";

/**
 * Window specification
 */
export type WindowSpec = { partition_by: Array<Expression>, order_by: Array<Ordered>, frame: WindowFrame | null, };
