// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Pseudocolumn type
 */
export type PseudocolumnType = "Rownum" | "Rowid" | "Level" | "Sysdate" | "ObjectId" | "ObjectValue";
