// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * COMMENT ON statement
 */
export type Comment = { 
/**
 * The object being commented on
 */
this: Expression, 
/**
 * The object kind (COLUMN, TABLE, DATABASE, etc.)
 */
kind: string, 
/**
 * The comment text expression
 */
expression: Expression, 
/**
 * IF EXISTS clause
 */
exists: boolean, 
/**
 * MATERIALIZED keyword
 */
materialized: boolean, };
