// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * LockingProperty
 */
export type LockingProperty = { this: Expression | null, kind: string, for_or_in: Expression | null, lock_type: Expression | null, override_: Expression | null, };
