// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Identifier } from "./Identifier";
import type { UseKind } from "./UseKind";

/**
 * USE statement (USE database, USE ROLE, USE WAREHOUSE, USE CATALOG, USE SCHEMA)
 */
export type Use = { 
/**
 * The kind of object (DATABASE, SCHEMA, ROLE, WAREHOUSE, CATALOG, or None for default)
 */
kind: UseKind | null, 
/**
 * The name of the object
 */
this: Identifier, };
