// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { IntervalUnit } from "./IntervalUnit";

/**
 * DATEDIFF function
 */
export type DateDiffFunc = { this: Expression, expression: Expression, unit: IntervalUnit | null, };
