// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * OUTPUT clause (TSQL) - used in INSERT, UPDATE, DELETE
 */
export type OutputClause = { 
/**
 * Columns/expressions to output
 */
columns: Array<Expression>, 
/**
 * Optional INTO target table or table variable
 */
into_table: Expression | null, };
