// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Element in an EXCLUDE constraint: expression WITH operator
 */
export type ExcludeElement = { 
/**
 * The column expression (may include operator class, ordering, nulls)
 */
expression: string, 
/**
 * The operator (e.g., &&, =)
 */
operator: string, };
