// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Sequence property tag for ordering
 */
export type SeqPropKind = "Start" | "Increment" | "Minvalue" | "Maxvalue" | "Cache" | "Cycle" | "NoCycle" | "OwnedBy" | "Order" | "NoOrder" | "Comment";
