// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Identifier } from "./Identifier";

/**
 * Database option
 */
export type DatabaseOption = { "CharacterSet": string } | { "Collate": string } | { "Owner": Identifier } | { "Template": Identifier } | { "Encoding": string } | { "Location": string };
