// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * XMLTable
 */
export type XMLTable = { this: Expression, namespaces: Expression | null, passing: Expression | null, columns: Array<Expression>, by_ref: Expression | null, };
