// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * ARRAY constructor
 */
export type ArrayConstructor = { expressions: Array<Expression>, bracket_notation: boolean, 
/**
 * True if LIST keyword was used instead of ARRAY (DuckDB)
 */
use_list_keyword: boolean, };
