// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * OVERLAY function - OVERLAY(string PLACING replacement FROM position [FOR length])
 */
export type OverlayFunc = { this: Expression, replacement: Expression, from: Expression, length: Expression | null, };
