// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { AlterViewAction } from "./AlterViewAction";
import type { TableRef } from "./TableRef";
import type { ViewColumn } from "./ViewColumn";

/**
 * ALTER VIEW statement
 */
export type AlterView = { name: TableRef, actions: Array<AlterViewAction>, 
/**
 * MySQL: ALGORITHM = MERGE|TEMPTABLE|UNDEFINED
 */
algorithm?: string | null, 
/**
 * MySQL: DEFINER = 'user'@'host'
 */
definer?: string | null, 
/**
 * MySQL: SQL SECURITY = DEFINER|INVOKER
 */
sql_security?: string | null, 
/**
 * TSQL: WITH option (SCHEMABINDING, ENCRYPTION, VIEW_METADATA)
 */
with_option?: string | null, 
/**
 * Hive: Column aliases with optional comments: (c1 COMMENT 'text', c2)
 */
columns?: Array<ViewColumn>, };
