// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * OnConflict
 */
export type OnConflict = { duplicate: Expression | null, expressions: Array<Expression>, action: Expression | null, conflict_keys: Expression | null, index_predicate: Expression | null, constraint: Expression | null, where_: Expression | null, };
