// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { Ordered } from "./Ordered";

/**
 * STRING_AGG function (PostgreSQL/Standard SQL)
 */
export type StringAggFunc = { this: Expression, separator: Expression | null, order_by: Array<Ordered> | null, distinct: boolean, filter: Expression | null, 
/**
 * BigQuery LIMIT inside STRING_AGG
 */
limit?: Expression | null, };
