// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * WidthBucket
 */
export type WidthBucket = { this: Expression, min_value: Expression | null, max_value: Expression | null, num_buckets: Expression | null, threshold: Expression | null, };
