// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * JSONExtractScalar
 */
export type JSONExtractScalar = { this: Expression, expression: Expression, only_json_types: Expression | null, expressions: Array<Expression>, json_type: Expression | null, scalar_only: Expression | null, };
