// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * Merge
 */
export type Merge = { this: Expression, using: Expression, on: Expression | null, using_cond: Expression | null, whens: Expression | null, with_: Expression | null, returning: Expression | null, };
