// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * LISTAGG ON OVERFLOW behavior
 */
export type ListAggOverflow = "Error" | { "Truncate": { filler: Expression | null, with_count: boolean, } };
