// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { HintExpression } from "./HintExpression";

/**
 * Query hint
 */
export type Hint = { expressions: Array<HintExpression>, };
