// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { FunctionSetValue } from "./FunctionSetValue";

/**
 * A SET option in CREATE FUNCTION (PostgreSQL)
 */
export type FunctionSetOption = { name: string, value: FunctionSetValue, };
