// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Ordered } from "./Ordered";

/**
 * Represent an ORDER BY clause containing one or more sort specifications.
 */
export type OrderBy = { 
/**
 * The sort specifications, each with direction and null ordering.
 */
expressions: Array<Ordered>, 
/**
 * Whether this is ORDER SIBLINGS BY (Oracle hierarchical queries)
 */
siblings?: boolean, };
