// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { Identifier } from "./Identifier";

/**
 * XML Namespace declaration
 */
export type XmlNamespace = { this: Expression, alias: Identifier | null, };
