// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * TABLE ref or MODEL ref used as a function argument (BigQuery)
 * e.g., GAP_FILL(TABLE device_data, ...) or ML.PREDICT(MODEL mydataset.mymodel, ...)
 */
export type TableArgument = { 
/**
 * The keyword prefix: "TABLE" or "MODEL"
 */
prefix: string, 
/**
 * The table/model reference expression
 */
this: Expression, };
