// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { IntervalUnitSpec } from "./IntervalUnitSpec";

/**
 * Interval expression
 */
export type Interval = { 
/**
 * The value expression (e.g., '1', 5, column_ref)
 */
this: Expression | null, 
/**
 * The unit specification (optional - can be None, a simple unit, a span, or an expression)
 */
unit: IntervalUnitSpec | null, };
