// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * OFFSET clause
 */
export type Offset = { this: Expression, 
/**
 * Whether ROW/ROWS keyword was used (SQL standard syntax)
 */
rows?: boolean | null, };
