// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * SQL Command (COMMIT, ROLLBACK, BEGIN, etc.)
 */
export type Command = { 
/**
 * The command text (e.g., "ROLLBACK", "COMMIT", "BEGIN")
 */
this: string, };
