// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * AutoIncrementColumnConstraint - MySQL/TSQL auto-increment marker
 * TSQL: outputs "IDENTITY"
 */
export type AutoIncrementColumnConstraint = null;
