// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * The value of a SET option
 */
export type FunctionSetValue = { "Value": { value: string, use_to: boolean, } } | "FromCurrent";
