// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { Identifier } from "./Identifier";

/**
 * Domain constraint
 */
export type DomainConstraint = { name: Identifier | null, check: Expression, };
