// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * ARRAY_SORT function
 */
export type ArraySortFunc = { this: Expression, comparator: Expression | null, desc: boolean, nulls_first: boolean | null, };
