// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * IS NULL predicate
 */
export type IsNull = { this: Expression, not: boolean, 
/**
 * Whether this was the postfix form (ISNULL/NOTNULL) vs standard (IS NULL/IS NOT NULL)
 */
postfix_form: boolean, };
