// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { TableRef } from "./TableRef";

/**
 * DROP TABLE statement
 */
export type DropTable = { names: Array<TableRef>, if_exists: boolean, cascade: boolean, 
/**
 * Oracle: CASCADE CONSTRAINTS
 */
cascade_constraints: boolean, 
/**
 * Oracle: PURGE
 */
purge: boolean, };
