// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * WithFill
 */
export type WithFill = { from_: Expression | null, to: Expression | null, step: Expression | null, interpolate: Expression | null, };
