// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { TrimPosition } from "./TrimPosition";

/**
 * TRIM function
 */
export type TrimFunc = { this: Expression, characters: Expression | null, position: TrimPosition, 
/**
 * Whether SQL standard syntax was used (TRIM(BOTH chars FROM str)) vs function syntax (TRIM(str))
 */
sql_standard_syntax: boolean, 
/**
 * Whether the position was explicitly specified (BOTH/LEADING/TRAILING) vs defaulted
 */
position_explicit: boolean, };
