// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { JsonUniqueKeys } from "./JsonUniqueKeys";

/**
 * IS JSON predicate (SQL standard)
 * Checks if a value is valid JSON
 */
export type IsJson = { this: Expression, 
/**
 * JSON type: VALUE, SCALAR, OBJECT, or ARRAY (None = just IS JSON)
 */
json_type: string | null, 
/**
 * Key uniqueness constraint
 */
unique_keys: JsonUniqueKeys | null, 
/**
 * Whether IS NOT JSON
 */
negated: boolean, };
