// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * Tags are used for generating arbitrary sql like SELECT <span>x</span>.
 */
export type Tag = { this: Expression | null, prefix: Expression | null, postfix: Expression | null, };
