// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Action for LIKE clause options
 */
export type LikeOptionAction = "Including" | "Excluding";
