// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * ROW FORMAT clause for Hive/Spark
 */
export type RowFormat = { delimited: boolean, fields_terminated_by: string | null, collection_items_terminated_by: string | null, map_keys_terminated_by: string | null, lines_terminated_by: string | null, null_defined_as: string | null, };
