// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";
import type { TableRef } from "./TableRef";

/**
 * Trigger body
 */
export type TriggerBody = { "Execute": { function: TableRef, args: Array<Expression>, } } | { "Block": string };
