// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * ToMap - Materialize-style map constructor
 * Can hold either:
 * - A SELECT subquery (MAP(SELECT 'a', 1))
 * - A struct with key=>value entries (MAP['a' => 1, 'b' => 2])
 */
export type ToMap = { 
/**
 * Either a Select subquery or a Struct containing PropertyEQ entries
 */
this: Expression, };
