// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { DataType } from "./DataType";
import type { Expression } from "./Expression";
import type { Identifier } from "./Identifier";
import type { ParameterMode } from "./ParameterMode";

/**
 * Function parameter
 */
export type FunctionParameter = { name: Identifier | null, data_type: DataType, mode: ParameterMode | null, default: Expression | null, };
