// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * SessionUser - MySQL/PostgreSQL SESSION_USER function
 */
export type SessionUser = null;
