// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * Represent a HAVING clause containing a predicate over aggregate results.
 */
export type Having = { 
/**
 * The filter predicate, typically involving aggregate functions.
 */
this: Expression, };
