// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * CURRENT_TIMESTAMP_LTZ - Snowflake local timezone timestamp
 */
export type CurrentTimestampLTZ = { precision: number | null, };
