// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * AnalyzeColumns - Redshift ANALYZE ... PREDICATE COLUMNS / ALL COLUMNS
 */
export type AnalyzeColumns = { 
/**
 * e.g., "PREDICATE COLUMNS" or "ALL COLUMNS"
 */
this: string, };
