// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Credentials for external access (S3, Azure, etc.)
 */
export type Credentials = { credentials: Array<[string, string]>, encryption: string | null, storage: string | null, };
