// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { RollupIndex } from "./RollupIndex";

/**
 * RollupProperty - StarRocks ROLLUP (index_name(col1, col2), ...)
 */
export type RollupProperty = { expressions: Array<RollupIndex>, };
