// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Enumerate all supported SQL join types.
 *
 * Covers the standard join types (INNER, LEFT, RIGHT, FULL, CROSS, NATURAL)
 * as well as dialect-specific variants: SEMI/ANTI joins, LATERAL joins,
 * CROSS/OUTER APPLY (TSQL), ASOF joins (DuckDB/Snowflake), ARRAY joins
 * (ClickHouse), STRAIGHT_JOIN (MySQL), and implicit comma-joins.
 */
export type JoinKind = "Inner" | "Left" | "Right" | "Full" | "Outer" | "Cross" | "Natural" | "NaturalLeft" | "NaturalRight" | "NaturalFull" | "Semi" | "Anti" | "LeftSemi" | "LeftAnti" | "RightSemi" | "RightAnti" | "CrossApply" | "OuterApply" | "AsOf" | "AsOfLeft" | "AsOfRight" | "Lateral" | "LeftLateral" | "Straight" | "Implicit" | "Array" | "LeftArray";
