// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * MATCH type for foreign keys
 */
export type MatchType = "Full" | "Partial" | "Simple";
