// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * AlterSet
 */
export type AlterSet = { expressions: Array<Expression>, option: Expression | null, tablespace: Expression | null, access_method: Expression | null, file_format: Expression | null, copy_options: Expression | null, tag: Expression | null, location: Expression | null, serde: Expression | null, };
