// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * SetOperation
 */
export type SetOperation = { with_: Expression | null, this: Expression, expression: Expression, distinct: boolean, by_name: Expression | null, side: Expression | null, kind: string | null, on: Expression | null, };
