// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * DateFromParts
 */
export type DateFromParts = { year: Expression | null, month: Expression | null, day: Expression | null, allow_overflow: Expression | null, };
