// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * SQL Comment preservation
 */
export type SqlComment = { text: string, is_block: boolean, };
