// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * HexStringExpr - Hex string expression (not literal)
 * BigQuery: converts to FROM_HEX(this)
 */
export type HexStringExpr = { this: Expression, is_integer: boolean | null, };
