// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { SetItem } from "./SetItem";

/**
 * SET variable statement
 */
export type SetStatement = { 
/**
 * The items being set
 */
items: Array<SetItem>, };
