// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Parameter mode (IN, OUT, INOUT)
 */
export type ParameterMode = "In" | "Out" | "InOut";
