// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { TableRef } from "./TableRef";

/**
 * DROP SEQUENCE statement
 */
export type DropSequence = { name: TableRef, if_exists: boolean, cascade: boolean, };
