// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { DataType } from "./DataType";
import type { Expression } from "./Expression";
import type { Identifier } from "./Identifier";

/**
 * Lambda expression
 */
export type LambdaExpr = { parameters: Array<Identifier>, body: Expression, 
/**
 * True if using DuckDB's LAMBDA x : expr syntax (vs x -> expr)
 */
colon: boolean, 
/**
 * Optional type annotations for parameters (Snowflake: a int -> a + 1)
 * Maps parameter index to data type
 */
parameter_types: Array<DataType | null>, };
