// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * LIKE/ILIKE operation with optional ESCAPE clause and quantifier (ANY/ALL)
 */
export type LikeOp = { left: Expression, right: Expression, 
/**
 * ESCAPE character/expression
 */
escape: Expression | null, 
/**
 * Quantifier: ANY, ALL, or SOME
 */
quantifier: string | null, };
