// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Parameter placeholder styles
 */
export type ParameterStyle = "Question" | "Dollar" | "DollarBrace" | "Brace" | "Colon" | "At" | "DoubleAt" | "DoubleDollar" | "Percent";
