// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * Individual hint expression
 */
export type HintExpression = { "Function": { name: string, args: Array<Expression>, } } | { "Identifier": string } | { "Raw": string };
