// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * TOP clause (SQL Server)
 */
export type Top = { this: Expression, percent: boolean, with_ties: boolean, 
/**
 * Whether the expression was parenthesized: TOP (10) vs TOP 10
 */
parenthesized: boolean, };
