// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { SequenceBound } from "./SequenceBound";
import type { TableRef } from "./TableRef";

/**
 * ALTER SEQUENCE statement
 */
export type AlterSequence = { name: TableRef, if_exists: boolean, increment: bigint | null, minvalue: SequenceBound | null, maxvalue: SequenceBound | null, start: bigint | null, restart: bigint | null | null, cache: bigint | null, cycle: boolean | null, owned_by: TableRef | null | null, };
