// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * KILL statement (MySQL/MariaDB)
 * KILL [CONNECTION | QUERY] <id>
 */
export type Kill = { 
/**
 * The target (process ID or connection ID)
 */
this: Expression, 
/**
 * Optional kind: "CONNECTION" or "QUERY"
 */
kind: string | null, };
