// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.

/**
 * Comparison operator for quantified expressions
 */
export type QuantifiedOp = "Eq" | "Neq" | "Lt" | "Lte" | "Gt" | "Gte";
