// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * NTH_VALUE function
 */
export type NthValueFunc = { this: Expression, offset: Expression, 
/**
 * None = not specified, Some(true) = IGNORE NULLS, Some(false) = RESPECT NULLS
 */
ignore_nulls?: boolean | null, };
