// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { Expression } from "./Expression";

/**
 * TimeFromParts
 */
export type TimeFromParts = { hour: Expression | null, min: Expression | null, sec: Expression | null, nano: Expression | null, fractions: Expression | null, precision: bigint | null, };
